@@ -137,10 +137,6 @@ void tangle_actor_send_event(TangleActor* actor, TangleEvent* event);
 void tangle_actor_handle_ancestor_event(TangleActor* actor, TangleEvent* event);
 void tangle_actor_handle_descendant_event(TangleActor* actor, TangleEvent* event);
 
-gchar* tangle_actor_lookup_filename(TangleActor* actor, const gchar* filename);
-ClutterScript* tangle_actor_get_script(TangleActor* actor);
-void _tangle_actor_set_script(TangleActor* actor, ClutterScript* script);
-
 TangleSpacing* tangle_spacing_new(gfloat top, gfloat right, gfloat bottom, gfloat left);
 TangleSpacing* tangle_spacing_copy(const TangleSpacing* spacing);
 void tangle_spacing_free(TangleSpacing* spacing);
