@@ -10,7 +10,7 @@
 #ifndef __TANGLE_BINDING_H__
 #define __TANGLE_BINDING_H__
 
-#include <glib-object.h>
+#include "tangle-object.h"
 
 #define TANGLE_TYPE_BINDING (tangle_binding_get_type())
 #define TANGLE_BINDING(obj) (G_TYPE_CHECK_INSTANCE_CAST((obj), TANGLE_TYPE_BINDING, TangleBinding))
@@ -19,6 +19,8 @@
 #define TANGLE_IS_BINDING_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass), TANGLE_TYPE_BINDING))
 #define TANGLE_BINDING_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS((obj), TANGLE_TYPE_BINDING, TangleBindingClass))
 
+#define TANGLE_TYPE_BINDING_DIRECTION (tangle_binding_direction_get_type())
+
 typedef struct _TangleBindingPrivate TangleBindingPrivate;
 typedef struct _TangleBinding TangleBinding;
 typedef struct _TangleBindingClass TangleBindingClass;
@@ -33,16 +35,18 @@ typedef enum _TangleBindingDirection {
 typedef void (*TangleBindingCallback)(GObject* object, const gchar* property, gpointer user_data);
 
 struct _TangleBinding {
-	GObject parent_instance;
+	TangleObject parent_instance;
 	TangleBindingPrivate* priv;
 };
 
 struct _TangleBindingClass {
-	GObjectClass parent_class;
+	TangleObjectClass parent_class;
 };
 
 GType tangle_binding_get_type(void) G_GNUC_CONST;
 
+GType tangle_binding_direction_get_type(void) G_GNUC_CONST;
+
 TangleBinding* tangle_binding_new(GObject* object_a, const gchar* property_a, GObject* object_b, const gchar* property_b);
 
 GObject* tangle_binding_get_object_a(TangleBinding* binding);
@@ -50,6 +54,13 @@ const gchar* tangle_binding_get_property_a(TangleBinding* binding);
 GObject* tangle_binding_get_object_b(TangleBinding* binding);
 const gchar* tangle_binding_get_property_b(TangleBinding* binding);
 TangleBindingDirection tangle_binding_get_direction(TangleBinding* binding);
+
+gboolean tangle_binding_get_multiplier(TangleBinding* binding, GValue* value_return);
+void tangle_binding_set_multiplier(TangleBinding* binding, const GValue* value);
+
+gboolean tangle_binding_get_constant_term(TangleBinding* binding, GValue* value_return);
+void tangle_binding_set_constant_term(TangleBinding* binding, const GValue* value);
+
 GList* tangle_object_get_bindings_readonly(GObject* object, const gchar* property);
 
 #endif
