@@ -41,7 +41,7 @@ struct _TangleButtonPrivate {
 	ClutterActor* selected_background_actor;
 	TangleAction* clicked_action;
 
-	guint button_release_event_handler_id;
+	ClutterActor* button_release_event_handler_actor;
 
 	guint active : 1;
 	guint inside : 1;
@@ -55,6 +55,7 @@ static const GEnumValue button_mode_values[] = {
 	{ 0, NULL, NULL }
 };
 
+static gboolean on_button_release_event(ClutterActor* actor, ClutterEvent* event, gpointer user_data);
 static void begin_interacting(TangleButton* button);
 static void end_interacting(TangleButton* button);
 
@@ -147,7 +148,9 @@ void tangle_button_set_normal_background_actor(TangleButton* button, ClutterActo
 		button->priv->normal_background_actor = actor;
 		if (button->priv->normal_background_actor) {
 			g_object_ref_sink(button->priv->normal_background_actor);
+			clutter_actor_push_internal(CLUTTER_ACTOR(button));
 			clutter_actor_set_parent(button->priv->normal_background_actor, CLUTTER_ACTOR(button));	
+			clutter_actor_pop_internal(CLUTTER_ACTOR(button));
 		}
 
 		if (!button->priv->active || !button->priv->interactive_background_actor) {
@@ -176,7 +179,9 @@ void tangle_button_set_interactive_background_actor(TangleButton* button, Clutte
 		button->priv->interactive_background_actor = actor;
 		if (button->priv->interactive_background_actor) {
 			g_object_ref_sink(button->priv->interactive_background_actor);
+			clutter_actor_push_internal(CLUTTER_ACTOR(button));
 			clutter_actor_set_parent(button->priv->interactive_background_actor, CLUTTER_ACTOR(button));	
+			clutter_actor_pop_internal(CLUTTER_ACTOR(button));
 		}
 
 		if (button->priv->active) {
@@ -209,7 +214,9 @@ void tangle_button_set_selected_background_actor(TangleButton* button, ClutterAc
 		button->priv->selected_background_actor = actor;
 		if (button->priv->selected_background_actor) {
 			g_object_ref_sink(button->priv->selected_background_actor);
+			clutter_actor_push_internal(CLUTTER_ACTOR(button));
 			clutter_actor_set_parent(button->priv->selected_background_actor, CLUTTER_ACTOR(button));	
+			clutter_actor_pop_internal(CLUTTER_ACTOR(button));
 		}
 
 		if (button->priv->selected) {
@@ -359,14 +366,14 @@ static void tangle_button_dispose(GObject* object) {
 	
 	button = TANGLE_BUTTON(object);
 	
-	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(button->priv->normal_background_actor);
-	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(button->priv->interactive_background_actor);
-	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(button->priv->selected_background_actor);
-	TANGLE_UNREF_AND_NULLIFY_OBJECT_POINTER(button->priv->clicked_action);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT(button->priv->normal_background_actor);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT(button->priv->interactive_background_actor);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT(button->priv->selected_background_actor);
+	TANGLE_UNREF_AND_NULLIFY_OBJECT(button->priv->clicked_action);
 	
-	if (button->priv->button_release_event_handler_id) {
-		g_signal_handler_disconnect(clutter_actor_get_stage(CLUTTER_ACTOR(button)), button->priv->button_release_event_handler_id);
-		button->priv->button_release_event_handler_id = 0;
+	if (button->priv->button_release_event_handler_actor) {
+		g_signal_handlers_disconnect_by_func(button->priv->button_release_event_handler_actor, G_CALLBACK(on_button_release_event), button);
+		button->priv->button_release_event_handler_actor = NULL;
 	}
 
 	G_OBJECT_CLASS(tangle_button_parent_class)->dispose(object);
@@ -457,8 +464,6 @@ static void tangle_button_class_init(TangleButtonClass* button_class) {
 	g_type_class_add_private(gobject_class, sizeof(TangleButtonPrivate));
 }
 
-static gboolean on_button_release_event(ClutterActor* actor, ClutterEvent* event, gpointer user_data);
-
 static void start_interacting(TangleButton* button) {
 	TangleEvent* event;
 
@@ -467,7 +472,10 @@ static void start_interacting(TangleButton* button) {
 	if (button->priv->interactive_background_actor) {
 		tangle_widget_change_background_actor(TANGLE_WIDGET(button), button->priv->interactive_background_actor);
 	}
-	button->priv->button_release_event_handler_id = g_signal_connect(clutter_actor_get_stage(CLUTTER_ACTOR(button)), "button-release-event", G_CALLBACK(on_button_release_event), button);
+	
+	button->priv->button_release_event_handler_actor = clutter_actor_get_stage(CLUTTER_ACTOR(button));
+	g_object_add_weak_pointer(G_OBJECT(button->priv->button_release_event_handler_actor), (gpointer*)&button->priv->button_release_event_handler_actor);
+	g_signal_connect(button->priv->button_release_event_handler_actor, "button-release-event", G_CALLBACK(on_button_release_event), button);
 
 	tangle_actor_send_interaction_began_event(TANGLE_ACTOR(button), TRUE, TRUE, FALSE);
 }
@@ -509,9 +517,9 @@ static gboolean on_button_release_event(ClutterActor* actor, ClutterEvent* event
 		g_signal_emit(button, signals[CLICKED], 0, &handled);
 	}
 
-	if (button->priv->button_release_event_handler_id) {
-		g_signal_handler_disconnect(clutter_actor_get_stage(CLUTTER_ACTOR(button)), button->priv->button_release_event_handler_id);
-		button->priv->button_release_event_handler_id = 0;
+	if (button->priv->button_release_event_handler_actor) {
+		g_signal_handlers_disconnect_by_func(button->priv->button_release_event_handler_actor, G_CALLBACK(on_button_release_event), button);
+		button->priv->button_release_event_handler_actor = NULL;
 	}
 
 	return FALSE;
