@@ -8,6 +8,7 @@
  */
 
 #include "tangle-draggable-actor.h"
+#include "tangle-clone.h"
 #include "marshalers.h"
 
 /**
@@ -196,22 +197,28 @@ static gboolean tangle_draggable_actor_end_dragging(TangleDraggableActor* dragga
 
 static gboolean tangle_draggable_actor_button_press_event(ClutterActor* actor, ClutterButtonEvent* event) {
 	TangleDraggableActor* draggable_actor;
-	gboolean handled;
+	ClutterActor* real_actor;
 	gfloat x, y;
 	gfloat width, height;
 	
 	draggable_actor = TANGLE_DRAGGABLE_ACTOR(actor);
 	
 	if (draggable_actor->priv->dragging_enabled) {
-		clutter_actor_get_transformed_position(actor, &x, &y);
 		draggable_actor->priv->dragging = tangle_dragging_new(draggable_actor);
+		if ((real_actor = clutter_stage_get_actor_at_pos(clutter_event_get_stage((ClutterEvent*)event), CLUTTER_PICK_ALL, event->x, event->y)) != actor &&
+		    TANGLE_IS_CLONE(real_actor)) {
+			draggable_actor->priv->dragging->real_draggable_actor = real_actor;
+		} else {
+			draggable_actor->priv->dragging->real_draggable_actor = actor;
+		}
+		clutter_actor_get_transformed_position(draggable_actor->priv->dragging->real_draggable_actor, &x, &y);
 		draggable_actor->priv->dragging->x = event->x;
 		draggable_actor->priv->dragging->y = event->y;
 		draggable_actor->priv->dragging->anchor_x = event->x - x;
 		draggable_actor->priv->dragging->anchor_y = event->y - y;
 		draggable_actor->priv->dragging->dx = event->x - x - draggable_actor->priv->dragging->anchor_x;
 		draggable_actor->priv->dragging->dy = event->y - y - draggable_actor->priv->dragging->anchor_y;
-		clutter_actor_get_size(actor, &width, &height);
+		clutter_actor_get_size(draggable_actor->priv->dragging->real_draggable_actor, &width, &height);
 		draggable_actor->priv->dragging->center_x = x + width / 2;
 		draggable_actor->priv->dragging->center_y = y + height / 2;
 
@@ -430,10 +437,10 @@ static gboolean on_captured_event(ClutterActor* actor, ClutterEvent* event, gpoi
 		g_object_ref(draggable_actor);
 
 		clutter_event_get_coords(event, &draggable_actor->priv->dragging->x, &draggable_actor->priv->dragging->y);
-		clutter_actor_get_transformed_position(CLUTTER_ACTOR(draggable_actor), &x, &y);
+		clutter_actor_get_transformed_position(draggable_actor->priv->dragging->real_draggable_actor, &x, &y);
 		draggable_actor->priv->dragging->dx = draggable_actor->priv->dragging->x - x - draggable_actor->priv->dragging->anchor_x;
 		draggable_actor->priv->dragging->dy = draggable_actor->priv->dragging->y - y - draggable_actor->priv->dragging->anchor_y;
-		clutter_actor_get_size(CLUTTER_ACTOR(draggable_actor), &width, &height);
+		clutter_actor_get_size(draggable_actor->priv->dragging->real_draggable_actor, &width, &height);
 		draggable_actor->priv->dragging->center_x = x + width / 2 + draggable_actor->priv->dragging->dx;
 		draggable_actor->priv->dragging->center_y = y + height / 2 + draggable_actor->priv->dragging->dy;
 
