@@ -13,18 +13,17 @@
 #define TANGLE_TYPE_EVENT (tangle_event_get_type())
 #define TANGLE_EVENT(e) ((TangleEvent*)(e))
 
-typedef enum _TangleEventType TangleEventType;
 typedef struct _TangleAnyEvent TangleAnyEvent;
 typedef struct _TangleInteractionEvent TangleInteractionEvent;
 typedef union _TangleEvent TangleEvent;
 
-#include "tangle-actor.h"
-
-enum _TangleEventType {
+typedef enum _TangleEventType {
 	TANGLE_EVENT_NONE = 0,
 	TANGLE_EVENT_INTERACTION_BEGAN,
 	TANGLE_EVENT_INTERACTION_ENDED
-};
+} TangleEventType;
+
+#include "tangle-actor.h"
 
 struct _TangleAnyEvent {
 	TangleEventType type;
