@@ -24,28 +24,33 @@
  */
 
 static GHashTable* calculation_functions_hash_table = NULL;
+static gchar** search_paths = { NULL };
 
 static void on_weak_notify(gpointer user_data, GObject* other_object);
 
 #define DEFINE_CALCULATION_FUNCTIONS(t) \
-	static void add_##t(GValue* value, GValue* addend) { g##t v; g##t a; \
+	static gint compare_##t(const GValue* value, const GValue* comparable) { g##t v; g##t c; \
+	v = g_value_get_##t(value); c = g_value_get_##t(comparable); \
+	return (v == c ? 0 : (v < c ? -1 : 1)); \
+	} \
+	static void add_##t(GValue* value, const GValue* addend) { g##t v; g##t a; \
 	v = g_value_get_##t(value); a = g_value_get_##t(addend); \
 	v += a; g_value_set_##t(value, v); \
 	} \
-	static void substract_##t(GValue* value, GValue* addend) { g##t v; g##t a; \
-	v = g_value_get_##t(value); a = g_value_get_##t(addend); \
-	v -= a; g_value_set_##t(value, v); \
+	static void substract_##t(GValue* value, const GValue* substrahend) { g##t v; g##t s; \
+	v = g_value_get_##t(value); s = g_value_get_##t(substrahend); \
+	v -= s; g_value_set_##t(value, v); \
 	} \
-	static void multiply_##t(GValue* value, GValue* addend) { g##t v; g##t a; \
-	v = g_value_get_##t(value); a = g_value_get_##t(addend); \
-	v *= a; g_value_set_##t(value, v); \
+	static void multiply_##t(GValue* value, const GValue* multiplier) { g##t v; g##t m; \
+	v = g_value_get_##t(value); m = g_value_get_##t(multiplier); \
+	v *= m; g_value_set_##t(value, v); \
 	} \
-	static void divide_##t(GValue* value, GValue* addend) { g##t v; g##t a; \
-	v = g_value_get_##t(value); a = g_value_get_##t(addend); \
-	v /= a; g_value_set_##t(value, v); \
+	static void divide_##t(GValue* value, const GValue* divider) { g##t v; g##t d; \
+	v = g_value_get_##t(value); d = g_value_get_##t(divider); \
+	v /= d; g_value_set_##t(value, v); \
 	} \
 	static TangleValueCalculationFunctions calculation_functions_##t = { \
-	add_##t, substract_##t, multiply_##t, divide_##t };
+	compare_##t, add_##t, substract_##t, multiply_##t, divide_##t };
 
 DEFINE_CALCULATION_FUNCTIONS(char)
 DEFINE_CALCULATION_FUNCTIONS(uchar)
@@ -327,7 +332,24 @@ gboolean tangle_signal_connect_dynamically_from_script(GObject* object, const gc
 	return retvalue;
 }
 
-void tangle_value_add(GValue* value, GValue* addend) {
+gint tangle_value_compare(const GValue* value, const GValue* comparable) {
+	gint retvalue = 0;
+	TangleValueCalculationFunctions* calculation_functions;
+	
+	g_return_if_fail(G_VALUE_TYPE(value) == G_VALUE_TYPE(comparable));
+	
+	if (!calculation_functions_hash_table || !(calculation_functions = (TangleValueCalculationFunctions*)g_hash_table_lookup(calculation_functions_hash_table, GUINT_TO_POINTER(G_VALUE_TYPE(value))))) {
+		g_critical("No calculation functions for value type '%s'.", G_VALUE_TYPE_NAME(value));
+	} else if (!calculation_functions->compare) {
+		g_critical("No compare function for value type '%s'.", G_VALUE_TYPE_NAME(value));	
+	} else {
+		retvalue = calculation_functions->compare(value, comparable);
+	}
+	
+	return retvalue;
+}
+
+void tangle_value_add(GValue* value, const GValue* addend) {
 	TangleValueCalculationFunctions* calculation_functions;
 	
 	g_return_if_fail(G_VALUE_TYPE(value) == G_VALUE_TYPE(addend));
@@ -341,7 +363,7 @@ void tangle_value_add(GValue* value, GValue* addend) {
 	}
 }
 
-void tangle_value_substract(GValue* value, GValue* subtrahend) {
+void tangle_value_substract(GValue* value, const GValue* subtrahend) {
 	TangleValueCalculationFunctions* calculation_functions;
 	
 	g_return_if_fail(G_VALUE_TYPE(value) == G_VALUE_TYPE(subtrahend));
@@ -355,7 +377,7 @@ void tangle_value_substract(GValue* value, GValue* subtrahend) {
 	}
 }
 
-void tangle_value_multiply(GValue* value, GValue* multiplier) {
+void tangle_value_multiply(GValue* value, const GValue* multiplier) {
 	TangleValueCalculationFunctions* calculation_functions;
 	
 	g_return_if_fail(G_VALUE_TYPE(value) == G_VALUE_TYPE(multiplier));
@@ -369,7 +391,7 @@ void tangle_value_multiply(GValue* value, GValue* multiplier) {
 	}
 }
 
-void tangle_value_divide(GValue* value, GValue* divider) {
+void tangle_value_divide(GValue* value, const GValue* divider) {
 	TangleValueCalculationFunctions* calculation_functions;
 	
 	g_return_if_fail(G_VALUE_TYPE(value) == G_VALUE_TYPE(divider));
@@ -383,6 +405,16 @@ void tangle_value_divide(GValue* value, GValue* divider) {
 	}
 }
 
+gboolean tangle_value_is_calculation_supported(GType value_type) {
+	gboolean retvalue = FALSE;
+	
+	if (calculation_functions_hash_table && g_hash_table_lookup(calculation_functions_hash_table, GUINT_TO_POINTER(value_type))) {
+		retvalue = TRUE;
+	}
+	
+	return retvalue;
+}
+
 void tangle_value_register_calculation_functions(GType value_type, TangleValueCalculationFunctions* calculation_functions) {
 	TangleValueCalculationFunctions* calculation_functions_copy;
 
@@ -398,6 +430,45 @@ void tangle_value_register_calculation_functions(GType value_type, TangleValueCa
 	}
 }
 
+gchar* tangle_lookup_filename(const gchar* filename) {
+	gchar* path = NULL;
+	gchar** csp;
+	gchar* dir;
+	
+	if (g_path_is_absolute(filename)) {
+		path = g_strdup(filename);
+	} else {
+		for (csp = search_paths; !path && *csp; csp++) {
+			path = g_build_filename(*csp, filename, NULL);
+			if (!g_file_test(path, G_FILE_TEST_EXISTS)) {
+				g_free(path);
+				path = NULL;
+			}
+		}
+		
+		if (!path) {
+			dir = g_get_current_dir();
+			path = g_build_filename(dir, filename, NULL);
+			if (!g_file_test(path, G_FILE_TEST_EXISTS)) {
+				g_free(path);
+				path = NULL;
+			}
+			g_free(dir);
+		}
+	}
+	
+	return path;
+}
+
+void tangle_add_search_path(const gchar* search_path) {
+	gsize search_paths_length;
+	
+	search_paths_length = (search_paths ? g_strv_length(search_paths) : 0);
+	search_paths = g_renew(gchar*, search_paths, search_paths_length + 2);
+	search_paths[search_paths_length] = g_strdup(search_path);
+	search_paths[search_paths_length + 1] = NULL;
+}
+
 static void on_weak_notify(gpointer user_data, GObject* other_object) {
 	TangleVault* vault;
 	GObject* object;
