@@ -8,6 +8,7 @@
  */
 
 #include "tangle-scrolling-actor.h"
+#include "tangle-clone.h"
 #include "tangle-vault.h"
 
 /**
@@ -73,6 +74,8 @@ struct _TangleScrollingActorPrivate {
 
 	gulong captured_event_handler_id;
 	ClutterAnimation* animation[2];
+	
+	ClutterActor* clones[8];
 
 	guint kinetic_scrolling : 1;
 
@@ -86,6 +89,7 @@ static gboolean on_captured_event(ClutterActor* actor, ClutterEvent* event, gpoi
 static gfloat get_length(TangleScrollingActor* scrolling_actor, Axis axis);
 static gfloat normalise_scrolling_offset(TangleScrollingActor* scrolling_actor, Axis axis, gfloat scrolling_offset, gfloat* delta_return);
 static void stop_interacting(TangleScrollingActor* scrolling_actor);
+static void setup_clones(TangleScrollingActor* scrolling_actor);
 
 ClutterActor* tangle_scrolling_actor_new(ClutterActor* wrapped) {
 
@@ -211,6 +215,8 @@ void tangle_scrolling_actor_set_continuous_x(TangleScrollingActor* scrolling_act
 		if (continuous_x) {
 			tangle_scrolling_actor_set_repeat_x(scrolling_actor, TRUE);
 		}
+		
+		setup_clones(scrolling_actor);
 
 		g_object_thaw_notify(G_OBJECT(scrolling_actor));
 	}
@@ -232,6 +238,8 @@ void tangle_scrolling_actor_set_continuous_y(TangleScrollingActor* scrolling_act
 			tangle_scrolling_actor_set_repeat_y(scrolling_actor, TRUE);
 		}
 
+		setup_clones(scrolling_actor);
+
 		g_object_thaw_notify(G_OBJECT(scrolling_actor));
 	}
 }
@@ -364,62 +372,122 @@ static void tangle_scrolling_actor_allocate_wrapped(TangleWrapperActor* wrapper_
 
 	clutter_actor_allocate(wrapped, &actor_box, flags);
 	
+	/* 0 1 2
+	   3 X 4
+	   5 6 7 */
+	   
+	if (scrolling_actor->priv->clones[0]) {
+		actor_box.x1 -= scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.x2 -= scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.y1 -= scrolling_actor->priv->scrolling_length[VERTICAL];
+		actor_box.y2 -= scrolling_actor->priv->scrolling_length[VERTICAL];
+
+		clutter_actor_allocate(scrolling_actor->priv->clones[0], &actor_box, flags);
+
+		actor_box.x1 += scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.x2 += scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.y1 += scrolling_actor->priv->scrolling_length[VERTICAL];
+		actor_box.y2 += scrolling_actor->priv->scrolling_length[VERTICAL];
+	}
+		
+	if (scrolling_actor->priv->clones[1]) {
+		actor_box.y1 -= scrolling_actor->priv->scrolling_length[VERTICAL];
+		actor_box.y2 -= scrolling_actor->priv->scrolling_length[VERTICAL];
+
+		clutter_actor_allocate(scrolling_actor->priv->clones[1], &actor_box, flags);
+
+		actor_box.y1 += scrolling_actor->priv->scrolling_length[VERTICAL];
+		actor_box.y2 += scrolling_actor->priv->scrolling_length[VERTICAL];
+	}
+
+	if (scrolling_actor->priv->clones[2]) {
+		actor_box.x1 += scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.x2 += scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.y1 -= scrolling_actor->priv->scrolling_length[VERTICAL];
+		actor_box.y2 -= scrolling_actor->priv->scrolling_length[VERTICAL];
+
+		clutter_actor_allocate(scrolling_actor->priv->clones[2], &actor_box, flags);
+
+		actor_box.x1 -= scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.x2 -= scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.y1 += scrolling_actor->priv->scrolling_length[VERTICAL];
+		actor_box.y2 += scrolling_actor->priv->scrolling_length[VERTICAL];
+	}
+
+	if (scrolling_actor->priv->clones[3]) {
+		actor_box.x1 -= scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.x2 -= scrolling_actor->priv->scrolling_length[HORIZONTAL];
+
+		clutter_actor_allocate(scrolling_actor->priv->clones[3], &actor_box, flags);
+
+		actor_box.x1 += scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.x2 += scrolling_actor->priv->scrolling_length[HORIZONTAL];
+	}
+
+	if (scrolling_actor->priv->clones[4]) {
+		actor_box.x1 += scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.x2 += scrolling_actor->priv->scrolling_length[HORIZONTAL];
+
+		clutter_actor_allocate(scrolling_actor->priv->clones[4], &actor_box, flags);
+
+		actor_box.x1 -= scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.x2 -= scrolling_actor->priv->scrolling_length[HORIZONTAL];
+	}
+
+	if (scrolling_actor->priv->clones[5]) {
+		actor_box.x1 -= scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.x2 -= scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.y1 += scrolling_actor->priv->scrolling_length[VERTICAL];
+		actor_box.y2 += scrolling_actor->priv->scrolling_length[VERTICAL];
+
+		clutter_actor_allocate(scrolling_actor->priv->clones[5], &actor_box, flags);
+
+		actor_box.x1 += scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.x2 += scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.y1 -= scrolling_actor->priv->scrolling_length[VERTICAL];
+		actor_box.y2 -= scrolling_actor->priv->scrolling_length[VERTICAL];
+	}
+
+	if (scrolling_actor->priv->clones[6]) {
+		actor_box.y1 += scrolling_actor->priv->scrolling_length[VERTICAL];
+		actor_box.y2 += scrolling_actor->priv->scrolling_length[VERTICAL];
+
+		clutter_actor_allocate(scrolling_actor->priv->clones[6], &actor_box, flags);
+
+		actor_box.y1 -= scrolling_actor->priv->scrolling_length[VERTICAL];
+		actor_box.y2 -= scrolling_actor->priv->scrolling_length[VERTICAL];
+	}
+
+	if (scrolling_actor->priv->clones[7]) {
+		actor_box.x1 += scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.x2 += scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.y1 += scrolling_actor->priv->scrolling_length[VERTICAL];
+		actor_box.y2 += scrolling_actor->priv->scrolling_length[VERTICAL];
+
+		clutter_actor_allocate(scrolling_actor->priv->clones[7], &actor_box, flags);
+
+		actor_box.x1 -= scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.x2 -= scrolling_actor->priv->scrolling_length[HORIZONTAL];
+		actor_box.y1 -= scrolling_actor->priv->scrolling_length[VERTICAL];
+		actor_box.y2 -= scrolling_actor->priv->scrolling_length[VERTICAL];
+	}
+	
 	scrolling_actor->priv->scrolling_length[HORIZONTAL] -= box->x2 - box->x1;
 	scrolling_actor->priv->scrolling_length[VERTICAL] -= box->y2 - box->y1;
 }
 
 static void tangle_scrolling_actor_paint_wrapped(TangleWrapperActor* wrapper_actor, ClutterActor* wrapped) {
 	TangleScrollingActor* scrolling_actor;
-	ClutterActorBox actor_box;
+	gint i;
 	
 	scrolling_actor = TANGLE_SCROLLING_ACTOR(wrapper_actor);
 
 	TANGLE_WRAPPER_ACTOR_CLASS(tangle_scrolling_actor_parent_class)->paint_wrapped(wrapper_actor, wrapped);
-	clutter_actor_get_allocation_box(CLUTTER_ACTOR(scrolling_actor), &actor_box);
-	
-	if (scrolling_actor->priv->repeat[HORIZONTAL] && scrolling_actor->priv->scrolling_offset[HORIZONTAL] < 0.0 &&
-	    scrolling_actor->priv->repeat[VERTICAL] && scrolling_actor->priv->scrolling_offset[VERTICAL] < 0.0) {
-		cogl_translate(-scrolling_actor->priv->scrolling_length[HORIZONTAL] - (actor_box.x2 - actor_box.x1), -scrolling_actor->priv->scrolling_length[VERTICAL] - (actor_box.y2 - actor_box.y1), 0.0);
-		TANGLE_WRAPPER_ACTOR_CLASS(tangle_scrolling_actor_parent_class)->paint_wrapped(wrapper_actor, wrapped);
-		cogl_translate(scrolling_actor->priv->scrolling_length[HORIZONTAL] + (actor_box.x2 - actor_box.x1), scrolling_actor->priv->scrolling_length[VERTICAL] + (actor_box.y2 - actor_box.y1), 0.0);
-	}
-	if (scrolling_actor->priv->repeat[HORIZONTAL] && scrolling_actor->priv->scrolling_offset[HORIZONTAL] < 0.0 &&
-	    scrolling_actor->priv->repeat[VERTICAL] && scrolling_actor->priv->scrolling_offset[VERTICAL] > scrolling_actor->priv->scrolling_length[VERTICAL]) {
-		cogl_translate(-scrolling_actor->priv->scrolling_length[HORIZONTAL] - (actor_box.x2 - actor_box.x1), scrolling_actor->priv->scrolling_length[VERTICAL] + (actor_box.y2 - actor_box.y1), 0.0);
-		TANGLE_WRAPPER_ACTOR_CLASS(tangle_scrolling_actor_parent_class)->paint_wrapped(wrapper_actor, wrapped);
-		cogl_translate(scrolling_actor->priv->scrolling_length[HORIZONTAL] + (actor_box.x2 - actor_box.x1), -scrolling_actor->priv->scrolling_length[VERTICAL] - (actor_box.y2 - actor_box.y1), 0.0);
-	}
-	if (scrolling_actor->priv->repeat[HORIZONTAL] && scrolling_actor->priv->scrolling_offset[HORIZONTAL] < 0.0) {
-		cogl_translate(-scrolling_actor->priv->scrolling_length[HORIZONTAL] - (actor_box.x2 - actor_box.x1), 0.0, 0.0);
-		TANGLE_WRAPPER_ACTOR_CLASS(tangle_scrolling_actor_parent_class)->paint_wrapped(wrapper_actor, wrapped);
-		cogl_translate(scrolling_actor->priv->scrolling_length[HORIZONTAL] + (actor_box.x2 - actor_box.x1), 0.0, 0.0);
-	}
-	if (scrolling_actor->priv->repeat[VERTICAL] && scrolling_actor->priv->scrolling_offset[VERTICAL] < 0.0) {
-		cogl_translate(0.0, -scrolling_actor->priv->scrolling_length[VERTICAL] - (actor_box.y2 - actor_box.y1), 0.0);
-		TANGLE_WRAPPER_ACTOR_CLASS(tangle_scrolling_actor_parent_class)->paint_wrapped(wrapper_actor, wrapped);
-		cogl_translate(0.0, scrolling_actor->priv->scrolling_length[VERTICAL] + (actor_box.y2 - actor_box.y1), 0.0);
-	}
-	if (scrolling_actor->priv->repeat[HORIZONTAL] && scrolling_actor->priv->scrolling_offset[HORIZONTAL] > scrolling_actor->priv->scrolling_length[HORIZONTAL] &&
-	    scrolling_actor->priv->repeat[VERTICAL] && scrolling_actor->priv->scrolling_offset[VERTICAL] > scrolling_actor->priv->scrolling_length[VERTICAL]) {
-		cogl_translate(scrolling_actor->priv->scrolling_length[HORIZONTAL] + (actor_box.x2 - actor_box.x1), scrolling_actor->priv->scrolling_length[VERTICAL] + (actor_box.y2 - actor_box.y1), 0.0);
-		TANGLE_WRAPPER_ACTOR_CLASS(tangle_scrolling_actor_parent_class)->paint_wrapped(wrapper_actor, wrapped);
-		cogl_translate(-scrolling_actor->priv->scrolling_length[HORIZONTAL] - (actor_box.x2 - actor_box.x1), -scrolling_actor->priv->scrolling_length[VERTICAL] - (actor_box.y2 - actor_box.y1), 0.0);
-	}
-	if (scrolling_actor->priv->repeat[HORIZONTAL] && scrolling_actor->priv->scrolling_offset[HORIZONTAL] > scrolling_actor->priv->scrolling_length[HORIZONTAL] &&
-	    scrolling_actor->priv->repeat[VERTICAL] && scrolling_actor->priv->scrolling_offset[VERTICAL] < 0.0) {
-		cogl_translate(scrolling_actor->priv->scrolling_length[HORIZONTAL] + (actor_box.x2 - actor_box.x1), -scrolling_actor->priv->scrolling_length[VERTICAL] - (actor_box.y2 - actor_box.y1), 0.0);
-		TANGLE_WRAPPER_ACTOR_CLASS(tangle_scrolling_actor_parent_class)->paint_wrapped(wrapper_actor, wrapped);
-		cogl_translate(-scrolling_actor->priv->scrolling_length[HORIZONTAL] - (actor_box.x2 - actor_box.x1), scrolling_actor->priv->scrolling_length[VERTICAL] + (actor_box.y2 - actor_box.y1), 0.0);
-	}
-	if (scrolling_actor->priv->repeat[HORIZONTAL] && scrolling_actor->priv->scrolling_offset[HORIZONTAL] > scrolling_actor->priv->scrolling_length[HORIZONTAL]) {
-		cogl_translate(scrolling_actor->priv->scrolling_length[HORIZONTAL] + (actor_box.x2 - actor_box.x1), 0.0, 0.0);
-		TANGLE_WRAPPER_ACTOR_CLASS(tangle_scrolling_actor_parent_class)->paint_wrapped(wrapper_actor, wrapped);
-		cogl_translate(-scrolling_actor->priv->scrolling_length[HORIZONTAL] - (actor_box.x2 - actor_box.x1), 0.0, 0.0);
-	}
-	if (scrolling_actor->priv->repeat[VERTICAL] && scrolling_actor->priv->scrolling_offset[VERTICAL] > scrolling_actor->priv->scrolling_length[VERTICAL]) {
-		cogl_translate(0.0, scrolling_actor->priv->scrolling_length[VERTICAL] + (actor_box.y2 - actor_box.y1), 0.0);
-		TANGLE_WRAPPER_ACTOR_CLASS(tangle_scrolling_actor_parent_class)->paint_wrapped(wrapper_actor, wrapped);
-		cogl_translate(0.0, -scrolling_actor->priv->scrolling_length[VERTICAL] - (actor_box.y2 - actor_box.y1), 0.0);
+
+	for (i = 0; i < 8; i++) {
+		if (scrolling_actor->priv->clones[i]) {
+			clutter_actor_paint(scrolling_actor->priv->clones[i]);
+		}
 	}
 }
 
@@ -472,6 +540,21 @@ static void tangle_scrolling_actor_clamp_scrolling_offset_y_impl(TangleScrolling
 	}
 }
 
+static void tangle_scrolling_actor_map(ClutterActor* actor) {
+	TangleScrollingActor* scrolling_actor;
+	gint i;
+	
+	scrolling_actor = TANGLE_SCROLLING_ACTOR(actor);
+	
+	CLUTTER_ACTOR_CLASS(tangle_scrolling_actor_parent_class)->map(actor);
+
+	for (i = 0; i < 8; i++) {
+		if (scrolling_actor->priv->clones[i]) {
+			clutter_actor_map(scrolling_actor->priv->clones[i]);
+		}
+	}
+}
+
 static void tangle_scrolling_actor_set_property(GObject* object, guint prop_id, const GValue* value, GParamSpec* pspec) {
 	TangleScrollingActor* scrolling_actor;
 	
@@ -598,6 +681,7 @@ static void tangle_scrolling_actor_class_init(TangleScrollingActorClass* scrolli
 	gobject_class->get_property = tangle_scrolling_actor_get_property;
 	
 	clutter_actor_class->button_press_event = tangle_scrolling_actor_button_press_event;
+	clutter_actor_class->map = tangle_scrolling_actor_map;
 	
 	actor_class->handle_descendant_event = tangle_scrolling_actor_handle_descendant_event;
 
@@ -1021,3 +1105,60 @@ static void stop_interacting(TangleScrollingActor* scrolling_actor) {
 		tangle_actor_send_interaction_ended_event(TANGLE_ACTOR(scrolling_actor), TRUE, TRUE, TRUE);
 	}
 }
+
+static void create_clone(ClutterActor** actor_p, ClutterActor* wrapped_actor, ClutterActor* parent_actor) {
+	if (!*actor_p) {
+		*actor_p = tangle_clone_new(wrapped_actor);
+		clutter_actor_set_parent(*actor_p, parent_actor);
+	}
+}
+
+static void destroy_clone(ClutterActor** actor_p) {
+	if (*actor_p) {
+		clutter_actor_unparent(*actor_p);
+		*actor_p = NULL;
+	}
+}
+
+static void setup_clones(TangleScrollingActor* scrolling_actor) {
+	ClutterActor* wrapped_actor;
+	ClutterActor* parent_actor;
+	
+	clutter_actor_push_internal(CLUTTER_ACTOR(scrolling_actor));
+	
+	wrapped_actor = tangle_wrapper_actor_get_wrapped(TANGLE_WRAPPER_ACTOR(scrolling_actor));
+	parent_actor = CLUTTER_ACTOR(scrolling_actor);
+	
+	if (scrolling_actor->priv->repeat[VERTICAL]) {
+		create_clone(&scrolling_actor->priv->clones[1], wrapped_actor, parent_actor);
+		create_clone(&scrolling_actor->priv->clones[6], wrapped_actor, parent_actor);
+	} else {
+		destroy_clone(&scrolling_actor->priv->clones[0]);
+		destroy_clone(&scrolling_actor->priv->clones[1]);
+		destroy_clone(&scrolling_actor->priv->clones[2]);
+		destroy_clone(&scrolling_actor->priv->clones[5]);
+		destroy_clone(&scrolling_actor->priv->clones[6]);
+		destroy_clone(&scrolling_actor->priv->clones[7]);
+	}
+	
+	if (scrolling_actor->priv->repeat[HORIZONTAL]) {
+		create_clone(&scrolling_actor->priv->clones[3], wrapped_actor, parent_actor);
+		create_clone(&scrolling_actor->priv->clones[4], wrapped_actor, parent_actor);
+	} else {
+		destroy_clone(&scrolling_actor->priv->clones[0]);
+		destroy_clone(&scrolling_actor->priv->clones[2]);
+		destroy_clone(&scrolling_actor->priv->clones[3]);
+		destroy_clone(&scrolling_actor->priv->clones[4]);
+		destroy_clone(&scrolling_actor->priv->clones[5]);
+		destroy_clone(&scrolling_actor->priv->clones[7]);
+	}
+	
+	if (scrolling_actor->priv->repeat[VERTICAL] && scrolling_actor->priv->repeat[HORIZONTAL]) {
+		create_clone(&scrolling_actor->priv->clones[0], wrapped_actor, parent_actor);
+		create_clone(&scrolling_actor->priv->clones[2], wrapped_actor, parent_actor);
+		create_clone(&scrolling_actor->priv->clones[5], wrapped_actor, parent_actor);
+		create_clone(&scrolling_actor->priv->clones[7], wrapped_actor, parent_actor);	
+	}
+	
+	clutter_actor_pop_internal(CLUTTER_ACTOR(scrolling_actor));	
+}
