@@ -27,7 +27,7 @@ GType clutter_script_get_type_from_class (const gchar *name);
 
 static void clutter_scriptable_iface_init(ClutterScriptableIface *iface);
 
-G_DEFINE_TYPE_WITH_CODE(TangleStyle, tangle_style, G_TYPE_OBJECT,
+G_DEFINE_TYPE_WITH_CODE(TangleStyle, tangle_style, TANGLE_TYPE_OBJECT,
                         G_IMPLEMENT_INTERFACE(CLUTTER_TYPE_SCRIPTABLE, clutter_scriptable_iface_init););
 
 enum {
@@ -52,6 +52,7 @@ struct _TangleStylePrivate {
 	guint for_subclasses : 1;
 };
 
+static ClutterScriptableIface* parent_scriptable_iface = NULL;
 static GQuark quark_object_style = 0;
 static GQuark quark_object_style_properties = 0;
 static guint signals[LAST_SIGNAL];
@@ -402,13 +403,21 @@ static void tangle_style_set_custom_property(ClutterScriptable* scriptable, Clut
 	}
 
 	if (g_object_class_find_property(G_OBJECT_CLASS(TANGLE_STYLE_GET_CLASS(style)), name)) {
-		g_object_set_property(G_OBJECT(style), name, value);
+		if (parent_scriptable_iface->set_custom_property) {
+			parent_scriptable_iface->set_custom_property(scriptable, script, name, value);
+		} else {
+			g_object_set_property(G_OBJECT(scriptable), name, value);
+		}
 	} else {
 		tangle_style_set_style_property(style, name, value);
 	}
 }
 
 static void clutter_scriptable_iface_init(ClutterScriptableIface *iface) {
+	if (!(parent_scriptable_iface = parent_scriptable_iface = g_type_interface_peek_parent (iface))) {
+		parent_scriptable_iface = g_type_default_interface_peek(CLUTTER_TYPE_SCRIPTABLE);
+	}
+
 	iface->set_custom_property = tangle_style_set_custom_property;
 }
 
