@@ -22,12 +22,12 @@
 typedef struct _TangleStylePrivate TangleStylePrivate;
 
 typedef struct _TangleStyle {
-	GObject parent_instance;
+	TangleObject parent_instance;
 	TangleStylePrivate* priv;
 } TangleStyle;
 
 typedef struct _TangleStyleClass {
-	GObjectClass parent_class;
+	TangleObjectClass parent_class;
 	
 	/* virtual functions */
 	gboolean (*get_style_property)(TangleStyle* style, const gchar* name, GValue* value);
