@@ -122,7 +122,7 @@ TangleStyle* tangle_stylesheet_get_style_for_object(TangleStylesheet* stylesheet
 	TangleStyle* style;
 	
 	/* TODO: This can be optimized so that compound style is created only when really needed. */
-	compound_style = tangle_compound_style_new_with_stylesheet(stylesheet);
+	compound_style = tangle_compound_style_new_with_stylesheet(G_OBJECT_TYPE(object), stylesheet);
 	for (style_in_list = stylesheet->priv->styles; style_in_list; style_in_list = style_in_list->next) {
 		style = TANGLE_STYLE(style_in_list->data);
 		if (tangle_style_is_for_object(style, object)) {
