@@ -39,6 +39,9 @@ struct _TangleTemplatePrivate {
 	GType object_type;
 	GHashTable* parameters;
 	
+	ClutterScript* script;
+	GList* child_templates;
+	
 	guint instance_counter;
 };
 
@@ -47,8 +50,12 @@ static gchar* str_replace(gchar* string, gsize position, gsize length, const gch
 static gboolean substitute_parameters(TangleTemplate* template, GValue* value);
 static void set_property(const gchar* name, const GValue* value, gpointer user_data);
 static void free_value(gpointer data);
-static gboolean apply_parameters(TangleTemplate* template, const gchar* name, GValue* value);
+static gboolean apply_parameters(TangleTemplate* template, const gchar* name, GValue* value, const gchar* id_prefix);
 static gboolean on_signal(gpointer user_data);
+static void add_children(TangleTemplate* template, GObject* object);
+void set_template_property(const gchar* name, const GValue* value, gpointer user_data);
+void add_template_parameter(gpointer key, gpointer value, gpointer user_data);
+void add_child_template(gpointer data, gpointer user_data);
 
 static ClutterScriptableIface* parent_scriptable_iface = NULL;
 
@@ -63,6 +70,20 @@ GType tangle_template_get_object_type(TangleTemplate* template) {
 }
 
 GObject* tangle_template_create_object(TangleTemplate* template) {
+	GObject* object;
+	const gchar* template_id;
+	gchar* id = NULL;
+
+	if ((template_id = clutter_get_script_id(G_OBJECT(template)))) {
+		id = g_strdup_printf("%s$$%d", template_id, template->priv->instance_counter);
+	}
+	object = tangle_template_create_object_with_id(template, id);
+	g_free(id);
+
+	return object;
+}
+
+GObject* tangle_template_create_object_with_id(TangleTemplate* template, const gchar* id) {
 	GObject* object = NULL;
 	gboolean error = FALSE;
 	guint n;
@@ -71,11 +92,8 @@ GObject* tangle_template_create_object(TangleTemplate* template) {
 	GArray* nonconstruct_parameters;
 	GArray* signal_parameters;
 	gpointer type_class;
-	ClutterScript* script;
 	guint i;
 	GParamSpec* param_spec;
-	const gchar* template_id;
-	gchar* id;
 	ClutterScriptable* scriptable;
 	ClutterScriptableIface* scriptable_iface;
 	GParameter* parameter;
@@ -89,7 +107,7 @@ GObject* tangle_template_create_object(TangleTemplate* template) {
 	for (i = 0; i < n; i++) {
 		if (parameters[i].name[0] == '@') {
 			g_array_append_val(signal_parameters, parameters[i]);
-		} else if (!apply_parameters(template, parameters[i].name, &parameters[i].value)) {
+		} else if (!apply_parameters(template, parameters[i].name, &parameters[i].value, id)) {
 			error = TRUE;
 			break;
 		} else if ((param_spec = g_object_class_find_property(type_class, parameters[i].name)) &&
@@ -104,25 +122,19 @@ GObject* tangle_template_create_object(TangleTemplate* template) {
 	if (!error) {
 		object = g_object_newv(template->priv->object_type, construct_parameters->len, (GParameter*)construct_parameters->data);
 		
-		script = tangle_object_get_script(TANGLE_OBJECT(template));
-		
 		template->priv->instance_counter++;
-		if (script) {
-			if ((template_id = clutter_get_script_id(G_OBJECT(template)))) {
-				id = g_strdup_printf("%s$%d", template_id, template->priv->instance_counter);
-				if (CLUTTER_IS_SCRIPTABLE(object)) {
-					clutter_scriptable_set_id(CLUTTER_SCRIPTABLE(object), id);
-					g_free(id);
-				} else {
-					g_object_set_data_full(object, "clutter-script-id", id, g_free);
-				}
-			}
-			if (TANGLE_IS_ACTOR(object)) {
-				_tangle_actor_set_script(TANGLE_ACTOR(object), script);
+		if (id) {
+			if (CLUTTER_IS_SCRIPTABLE(object)) {
+				clutter_scriptable_set_id(CLUTTER_SCRIPTABLE(object), id);
+			} else {
+				g_object_set_data_full(object, "clutter-script-id", g_strdup(id), g_free);
 			}
 		}
-		
-		if (!script ||
+
+		/* TODO: This message will be removed soon! */
+		g_print("Created an object '%s' from the template '%s'.\n", id, clutter_get_script_id(G_OBJECT(template)));
+
+		if (!template->priv->script ||
 		    !CLUTTER_IS_SCRIPTABLE(object) ||
 		    !(scriptable = CLUTTER_SCRIPTABLE(object)) ||
 		    !(scriptable_iface = CLUTTER_SCRIPTABLE_GET_IFACE(scriptable)) ||
@@ -132,22 +144,24 @@ GObject* tangle_template_create_object(TangleTemplate* template) {
 		for (i = 0; i < nonconstruct_parameters->len; i++) {
 			parameter = &g_array_index(nonconstruct_parameters, GParameter, i);
 			if (scriptable_iface) {
-				scriptable_iface->set_custom_property(scriptable, script, parameter->name, &parameter->value);
+				scriptable_iface->set_custom_property(scriptable, template->priv->script, parameter->name, &parameter->value);
 			} else {
 				g_object_set_property(object, parameter->name, &parameter->value);
 			}
 		}
-
+		
+		add_children(template, object);
+		
 		for (i = 0; i < signal_parameters->len; i++) {
 			parameter = &g_array_index(signal_parameters, GParameter, i);
 			if (!G_VALUE_HOLDS(&parameter->value, G_TYPE_STRING) || !substitute_parameters(template, &parameter->value)) {
-				g_warning("Failed to connect the signal handler property '%s' of an object created from the template '%s', skipped.", parameter->name + 1, clutter_get_script_id(G_OBJECT(template)));
-			} else if (script) {
-				if (!!tangle_signal_connect_dynamically_from_script(object, parameter[i].name + 1, g_value_get_string(&parameter->value), script)) {
-					g_warning("Failed to connect the signal handler property '%s' of an object created from the template '%s', skipped.", parameters->name + 1, clutter_get_script_id(G_OBJECT(template)));
+				g_warning("Invalid signal property '%s' of an object created from the template '%s', skipped.", parameter->name, clutter_get_script_id(G_OBJECT(template)));
+			} else if (template->priv->script) {
+				if (!!tangle_signal_connect_dynamically_from_script(object, parameter->name + 1, g_value_get_string(&parameter->value), template->priv->script)) {
+					g_warning("Failed to connect the signal '%s' of an object created from the template '%s' into function '%s', skipped.", parameter->name + 1, clutter_get_script_id(G_OBJECT(template)), g_value_get_string(&parameter->value));
 				}
-			} else if (!tangle_signal_connect_dynamically(object, parameter[i].name + 1, g_value_get_string(&parameter->value), NULL)) {
-				g_warning("Failed to connect signal handler property '%s' of an object, skipped.", parameter->name + 1);
+			} else if (!tangle_signal_connect_dynamically(object, parameter->name + 1, g_value_get_string(&parameter->value), NULL)) {
+				g_warning("Failed to connect the signal '%s' into function '%s', skipped.", parameter->name + 1, g_value_get_string(&parameter->value));
 			}		
 		}
 	}
@@ -171,6 +185,8 @@ void tangle_template_apply_properties(TangleTemplate* template, GObject* object)
 	vault = tangle_vault_new(4, TANGLE_TYPE_TEMPLATE, template, G_TYPE_OBJECT, object, G_TYPE_ULONG, (gulong)0, G_TYPE_UINT, (guint)0);
 	tangle_properties_foreach(TANGLE_PROPERTIES(template), set_property, vault);
 	tangle_vault_free(vault);
+
+	add_children(template, object);
 }
 
 void tangle_template_animate_properties(TangleTemplate* template, ClutterActor* actor, gulong mode, guint duration) {
@@ -180,8 +196,7 @@ void tangle_template_animate_properties(TangleTemplate* template, ClutterActor*
 	tangle_properties_foreach(TANGLE_PROPERTIES(template), set_property, vault);
 	tangle_vault_free(vault);
 }
-
-gulong tangle_template_apply_properties_on_signal(TangleTemplate* template, GObject* target_object, GObject* signal_object, const gchar* signal_name, gboolean signal_return) {
+ngle_template_apply_properties_on_signal(TangleTemplate* template, GObject* target_object, GObject* signal_object, const gchar* signal_name, gboolean signal_return) {
 	TangleVault* vault;
 	
 	vault = tangle_vault_new(5, TANGLE_TYPE_TEMPLATE, template, G_TYPE_OBJECT, target_object, G_TYPE_ULONG, (gulong)0, G_TYPE_UINT, (guint)0, G_TYPE_BOOLEAN, signal_return);
@@ -353,7 +368,7 @@ static void tangle_template_class_init(TangleTemplateClass* template_class) {
 	GObjectClass* gobject_class = G_OBJECT_CLASS(template_class);
 	TanglePropertiesClass* properties_class = TANGLE_PROPERTIES_CLASS(template_class);
 
-	/* TODO: Free hash table! */
+	/* TODO: Free hash table && unref script! */
 	gobject_class->set_property = tangle_template_set_property;
 	gobject_class->get_property = tangle_template_get_property;
 	
@@ -397,13 +412,41 @@ static gboolean tangle_template_parse_custom_node(ClutterScriptable* scriptable,
 	TangleTemplate* t;
 	GList* name_list;
 	GList* name_list_item;
+	JsonArray* array;
+	gint i;
 	
 	template = TANGLE_TEMPLATE(scriptable);
-	_tangle_object_set_script(TANGLE_OBJECT(template), script);
+
+	if (!template->priv->script) {
+		template->priv->script = script;
+		g_object_ref(template->priv->script);
+	}
 
 	if (name[0] == '@') {
 		retvalue = clutter_script_parse_node(script, value, name, node, NULL);
-	} else if (strcmp(name, "object-type") && strcmp(name, "object-type-name")) {
+	} else if (!strcmp(name, "child-templates")) {
+		name_list = NULL;
+
+		if (!g_type_is_a(template->priv->object_type, CLUTTER_TYPE_CONTAINER)) {
+			g_critical("Object of type '%s' is not ClutterContainer, but the 'child-template' property found, skipped.", g_type_name(template->priv->object_type));
+		} else if (!JSON_NODE_HOLDS_ARRAY(node)) {
+			g_critical("Expected an array in 'child-templates' node, skipped.");
+		} else {
+			array = json_node_get_array(node);
+			for (i = json_array_get_length(array) - 1; i >= 0; i--) {
+				if (!(s = json_array_get_string_element(array, i))) {
+					g_critical("A non-string element in 'child-templates' array, skipped.");
+				} else {
+					name_list = g_list_prepend(name_list, g_strdup(s));
+				}
+			}
+		}
+			
+		g_value_init(value, G_TYPE_POINTER);
+		g_value_set_pointer(value, name_list);
+
+		retvalue = TRUE;
+	} else if (strcmp(name, "object-type") && strcmp(name, "object-type-name") && strcmp(name, "template")) {
 		type_class = g_type_class_ref(template->priv->object_type);
 		if (!(param_spec = g_object_class_find_property(G_OBJECT_CLASS(type_class), name))) {
 			g_warning("Object of type '%s' does not have property named '%s', skipping.", g_type_name(template->priv->object_type), name);
@@ -442,12 +485,39 @@ static gboolean tangle_template_parse_custom_node(ClutterScriptable* scriptable,
 	return retvalue;
 }
 
+static void tangle_template_set_custom_property(ClutterScriptable* scriptable, ClutterScript* script, const gchar* name, const GValue* value) {
+	TangleTemplate* template;
+	TangleTemplate* super_template;
+	
+	template = TANGLE_TEMPLATE(scriptable);
+
+	if (!strcmp(name, "template")) {
+		super_template = TANGLE_TEMPLATE(g_value_get_object(value));
+
+		g_return_if_fail(TANGLE_IS_TEMPLATE(super_template));
+		g_return_if_fail(template->priv->object_type == super_template->priv->object_type);
+
+		tangle_properties_foreach(TANGLE_PROPERTIES(super_template), set_template_property, template);
+		g_hash_table_foreach(super_template->priv->parameters, add_template_parameter, template);
+		g_list_foreach(super_template->priv->child_templates, add_child_template, template);
+
+		template->priv->child_templates = g_list_reverse(template->priv->child_templates);	
+	} else if (!strcmp(name, "child-templates")) {
+		g_return_if_fail(G_VALUE_HOLDS(value, G_TYPE_POINTER));
+
+		template->priv->child_templates = (GList*)g_value_get_pointer(value);
+	} else {
+		parent_scriptable_iface->set_custom_property(scriptable, script, name, value);
+	}
+}
+
 static void clutter_scriptable_iface_init(ClutterScriptableIface* iface) {
 	if (!(parent_scriptable_iface = parent_scriptable_iface = g_type_interface_peek_parent (iface))) {
 		parent_scriptable_iface = g_type_default_interface_peek(CLUTTER_TYPE_SCRIPTABLE);
 	}
 	
 	iface->parse_custom_node = tangle_template_parse_custom_node;
+	iface->set_custom_property = tangle_template_set_custom_property;
 }
 
 static gboolean is_value_null(gpointer key, gpointer value, gpointer user_data) {
@@ -534,7 +604,6 @@ static void set_property(const gchar* name, const GValue* value, gpointer user_d
 	gulong mode;
 	guint duration;
 	GValue v = { 0 };
-	ClutterScript* script;
 	
 	vault = TANGLE_VAULT(user_data);
 	tangle_vault_get(vault, 4, TANGLE_TYPE_TEMPLATE, &template, G_TYPE_OBJECT, &object, G_TYPE_ULONG, &mode, G_TYPE_UINT, &duration);
@@ -544,14 +613,14 @@ static void set_property(const gchar* name, const GValue* value, gpointer user_d
 	if (name[0] == '@') {
 		if (!G_VALUE_HOLDS(&v, G_TYPE_STRING) || !substitute_parameters(template, &v)) {
 			g_warning("Failed to connect signal handler property '%s' of an object (id: %s).", name, clutter_get_script_id(G_OBJECT(object)));
-		} else if ((script = tangle_object_get_script(TANGLE_OBJECT(template)))) {
-			if (!tangle_signal_connect_dynamically_from_script(object, name + 1, g_value_get_string(&v), script)) {
+		} else if (template->priv->script) {
+			if (!tangle_signal_connect_dynamically_from_script(object, name + 1, g_value_get_string(&v), template->priv->script)) {
 				g_warning("Failed to connect the signal handler property '%s' of an object created from the template '%s', skipped.", name + 1, clutter_get_script_id(G_OBJECT(template)));
 			}
 		} else if (!tangle_signal_connect_dynamically(object, name + 1, g_value_get_string(&v), NULL)) {
 			g_warning("Failed to connect the signal handler property '%s' of an object, skipped.", name + 1);
 		}
-	} else if (apply_parameters(template, name, &v)) {
+	} else if (apply_parameters(template, name, &v, clutter_get_script_id(object))) {
 		if (!mode || !duration || !CLUTTER_IS_ACTOR(object)) {
 			g_object_set_property(object, name, &v);
 		} else if (TANGLE_IS_ACTOR(object)) {
@@ -570,7 +639,7 @@ static void free_value(gpointer data) {
 	}
 }
 
-static gboolean apply_parameters(TangleTemplate* template, const gchar* name, GValue* value) {
+static gboolean apply_parameters(TangleTemplate* template, const gchar* name, GValue* value, const gchar* id_prefix) {
 	gboolean retvalue = TRUE;
 	gpointer type_class;
 	GObject* object;
@@ -583,6 +652,7 @@ static gboolean apply_parameters(TangleTemplate* template, const gchar* name, GV
 	gchar* s;
 	GList* name_list;
 	GList* name_list_item;
+	gchar* id;
 
 	type_class = g_type_class_ref(template->priv->object_type);
 
@@ -590,7 +660,7 @@ static gboolean apply_parameters(TangleTemplate* template, const gchar* name, GV
 		if ((param_spec = g_object_class_find_property(G_OBJECT_CLASS(type_class), name)) && G_IS_PARAM_SPEC_OBJECT(param_spec)) {
 			/* Handle deferred object resolution. */
 			substitute_parameters(template, value);
-			if ((object = clutter_script_get_object(tangle_object_get_script(TANGLE_OBJECT(template)), g_value_get_string(value)))) {
+			if ((object = clutter_script_get_object(template->priv->script, g_value_get_string(value)))) {
 				if (TANGLE_IS_TEMPLATE(object) && !G_VALUE_HOLDS(value, TANGLE_TYPE_TEMPLATE)) {
 					/* Create objects from templates recursively. */
 					t = TANGLE_TEMPLATE(object);
@@ -607,7 +677,13 @@ static gboolean apply_parameters(TangleTemplate* template, const gchar* name, GV
 					}
 					g_list_free(name_list);
 					
-					object = tangle_template_create_object(t);
+					if (id_prefix) {
+						id = g_strconcat(id_prefix, "$", name, NULL);
+					} else {
+						id = NULL;
+					}
+					object = tangle_template_create_object_with_id(t, id);
+					g_free(id);
 					g_value_unset(value);
 					g_value_init(value, tangle_template_get_object_type(t));
 					g_value_set_instance(value, object);
@@ -671,3 +747,55 @@ static gboolean on_signal(gpointer user_data) {
 	
 	return retvalue;
 }
+
+static void add_children(TangleTemplate* template, GObject* object) {
+	GList* child_template;
+	gint i;
+	gchar* s;
+	GObject* o;
+	const gchar* object_id;
+	gchar* id;
+
+	for (child_template = template->priv->child_templates, i = 0; child_template; child_template = child_template->next, i++) {
+		s = g_strdup((gchar*)child_template->data);
+		/* TODO: substitute_parameters() */
+		if (!(o = clutter_script_get_object(template->priv->script, s))) {
+			g_critical("A template (id: %s) not found when adding a children from 'child-templates' array, skipped.", s);
+		} if (!TANGLE_IS_TEMPLATE(o)) {
+			g_critical("An object (id: %s) is not TangleTemplate as expected in 'child-templates' array, skipped.", s);
+		} else {
+			if ((object_id = clutter_get_script_id(object))) {
+				id = g_strdup_printf("%s$%d", object_id, i);
+			} else {
+				id = NULL;
+			}
+			clutter_container_add_actor(CLUTTER_CONTAINER(object), CLUTTER_ACTOR(tangle_template_create_object_with_id(TANGLE_TEMPLATE(o), id)));
+			g_free(id);
+		}
+		g_free(s);		
+	}
+
+}
+
+void set_template_property(const gchar* name, const GValue* value, gpointer user_data) {
+	tangle_properties_set_property(TANGLE_PROPERTIES(user_data), name, value);		
+}
+
+void add_template_parameter(gpointer key, gpointer value, gpointer user_data) {
+	TangleTemplate* template;
+	GValue* value_copy;
+	
+	template = TANGLE_TEMPLATE(user_data);
+
+	value_copy = g_value_init(g_new0(GValue, 1), G_VALUE_TYPE(value));
+	g_value_copy(value, value_copy);
+	g_hash_table_insert(template->priv->parameters, g_strdup(key), value_copy);
+}
+
+void add_child_template(gpointer data, gpointer user_data) {
+	TangleTemplate* template;
+	
+	template = TANGLE_TEMPLATE(user_data);
+	
+	template->priv->child_templates = g_list_prepend(template->priv->child_templates, g_strdup(data));
+}
