@@ -18,7 +18,8 @@
 
 static void clutter_scriptable_iface_init(ClutterScriptableIface *iface);
 
-G_DEFINE_TYPE(TangleView, tangle_view, TANGLE_TYPE_WIDGET);
+G_DEFINE_TYPE_WITH_CODE(TangleView, tangle_view, TANGLE_TYPE_WIDGET,
+                        G_IMPLEMENT_INTERFACE(CLUTTER_TYPE_SCRIPTABLE, clutter_scriptable_iface_init););
 
 enum {
 	PROP_0,
@@ -33,6 +34,8 @@ struct _TangleViewPrivate {
 	guint loaded : 1;
 };
 
+static ClutterScriptableIface* parent_scriptable_iface = NULL;
+
 static void add_orphans_to_container(TangleView* view);
 
 ClutterActor* tangle_view_new() {
@@ -60,7 +63,7 @@ void tangle_view_set_filename(TangleView* view, const gchar* filename) {
 			view->priv->script_id = 0;
 		}
 		g_free(view->priv->filename);
-		view->priv->filename = tangle_actor_lookup_filename(TANGLE_ACTOR(view), filename);
+		view->priv->filename = tangle_lookup_filename(filename);
 		if (!view->priv->filename) {
 			g_warning("%s: file '%s' not found.", G_STRLOC, filename);
 		}
@@ -80,7 +83,7 @@ void tangle_view_load_now(TangleView* view) {
 	g_return_if_fail(view->priv->filename != NULL);
 
 	if (!view->priv->loaded) {
-		if (!view->priv->script && !(view->priv->script = tangle_actor_get_script(TANGLE_ACTOR(view)))) {
+		if (!view->priv->script) {
 			view->priv->script = clutter_script_new();
 		}
 
@@ -207,6 +210,32 @@ static void tangle_view_init(TangleView* view) {
 	view->priv = G_TYPE_INSTANCE_GET_PRIVATE(view, TANGLE_TYPE_VIEW, TangleViewPrivate);
 }
 
+static gboolean tangle_view_parse_custom_node(ClutterScriptable* scriptable, ClutterScript* script, GValue* value, const gchar* name, JsonNode* node) {
+	gboolean retvalue = FALSE;
+	TangleView* view;
+	
+	view = TANGLE_VIEW(scriptable);
+	
+	if (!view->priv->script) {
+		view->priv->script = script;
+		g_object_ref(view->priv->script);
+	}
+	
+	if (parent_scriptable_iface->parse_custom_node) {
+		retvalue = parent_scriptable_iface->parse_custom_node(scriptable, script, value, name, node);
+	}
+	
+	return retvalue;
+}
+
+static void clutter_scriptable_iface_init(ClutterScriptableIface* iface) {
+	if (!(parent_scriptable_iface = g_type_interface_peek_parent (iface))) {
+		parent_scriptable_iface = g_type_default_interface_peek(CLUTTER_TYPE_SCRIPTABLE);
+	}
+	
+	iface->parse_custom_node = tangle_view_parse_custom_node;
+}
+
 static void add_orphans_to_container(TangleView* view) {
 	GList* objects;
 	GList* list;
