@@ -17,6 +17,7 @@
 #include "tangle-binding.h"
 #include "tangle-box-layout.h"
 #include "tangle-button.h"
+#include "tangle-button-group.h"
 #include "tangle-call-function-action.h"
 #include "tangle-clamp.h"
 #include "tangle-compound-style.h"
@@ -32,6 +33,7 @@
 #include "tangle-linear-gradient.h"
 #include "tangle-multiplexed-transition.h"
 #include "tangle-misc.h"
+#include "tangle-object.h"
 #include "tangle-properties.h"
 #include "tangle-scrollbar.h"
 #include "tangle-scroller.h"
