@@ -28,7 +28,13 @@ texture = tangle_linear_gradient_new(color1, color2);
 tangle_actor_animate(TANGLE_ACTOR(texture), CLUTTER_EASE_IN_SINE, 5000, "angle", 6.28, NULL);
 g_object_set(actor, "width", 44.0, "height", 99.0, NULL);
 	tangle_widget_set_background_actor(TANGLE_WIDGET(actor), texture);
-clutter_actor_show(texture);
+
+	color1 = clutter_color_new(0, 255, 255, 255);
+	color2 = clutter_color_new(255, 0, 255, 0);
+	texture = tangle_linear_gradient_new(color1, color2);
+	tangle_linear_gradient_set_angle(TANGLE_LINEAR_GRADIENT(texture), 3.14 / 2);
+	tangle_widget_set_foreground_actor(TANGLE_WIDGET(actor), texture);
+
 
 	text = clutter_text_new_with_text("Sans 10pt", message);
 //	clutter_container_add(CLUTTER_CONTAINER(actor), text, NULL);
@@ -39,14 +45,13 @@ clutter_actor_show(texture);
 static ClutterAnimation* animate_transition(TangleActor* actor, ClutterActorBox* current_box, ClutterActorBox* new_box) {
 	ClutterAnimation* animation = NULL;
 	gdouble move_x, move_y, scale_x, scale_y;
-	gulong mode;
-	guint duration;
+	gulong mode = 1;
+	guint duration = 1500;
 	
 	if (!current_box) {
 		g_object_get(actor,
 		             "transition-move-x", &move_x, "transition-move-y", &move_y,
 		             "transition-scale-x", &scale_x, "transition-scale-y", &scale_y,
-			     "transition-mode", &mode, "transition-duration", &duration,
 			     NULL);
 		if (!duration) {
 			duration = 1000;
@@ -121,12 +126,13 @@ int main(int argc, char** argv) {
 	
 	clutter_init(&argc, &argv);
 	
-//	tangle_actor_set_default_transition_mode(CLUTTER_EASE_OUT_SINE);
-//	tangle_actor_set_default_transition_duration(250);
-	
 	stage = clutter_stage_get_default();
 	g_object_set(G_OBJECT(stage), "width", 800.0, "height", 480.0, NULL);
 
+printf("%p\n", tangle_widget_new());
+
+	tangle_stylesheet_set_default(tangle_stylesheet_new_from_file("box-stylesheet"));
+
 	box = tangle_widget_new();
 	tangle_widget_set_layout(TANGLE_WIDGET(box), tangle_box_layout_new());
 //	g_object_set(G_OBJECT(box), "width", 600.0, "height", 100.0, NULL);
